"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const htmlparser2_1 = __importDefault(require("htmlparser2"));
var Fragment;
(function (Fragment) {
    Fragment[Fragment["Open"] = 0] = "Open";
    Fragment[Fragment["Close"] = 1] = "Close";
    Fragment[Fragment["Text"] = 2] = "Text";
    Fragment[Fragment["None"] = 3] = "None";
})(Fragment || (Fragment = {}));
const previousFragment = (fragments) => {
    const prev = fragments.length ? fragments[fragments.length - 1] : '';
    if (/\[$/.test(prev)) {
        return Fragment.Open;
    }
    else if (/\]$/.test(prev)) {
        return Fragment.Close;
    }
    else if (/^\s([A-Za-z]+\.)?text\s".*"$/.test(prev)) {
        return Fragment.Text;
    }
    return Fragment.None;
};
const attributesToString = (attribs, alias) => {
    const str = Object.entries(attribs)
        .map(([key, value]) => {
        // The Html.Attributes package uses type_
        if (key === 'type') {
            key = 'type_';
        }
        if (alias.length) {
            key = alias + '.' + key;
        }
        // Treat attributes without value as truthy boolean
        value = value ? `"${value}"` : 'True';
        return ' ' + key + ' ' + value;
    })
        .join(',');
    return str ? str + ' ' : '';
};
const convert = (html, options = {}) => {
    const { indent = 4, attributeAlias = '', htmlAlias = '' } = options;
    const spaces = (amount) => ' '.repeat(amount * indent);
    const fragments = [];
    let depth = 0;
    const parser = new htmlparser2_1.default.Parser({
        onopentag: (name, attribs) => {
            let open = '';
            switch (previousFragment(fragments)) {
                case Fragment.Open:
                    open += ' ';
                    break;
                case Fragment.Close:
                case Fragment.Text:
                    open += '\n' + spaces(depth) + ', ';
                    break;
            }
            depth++;
            if (htmlAlias.length) {
                open += htmlAlias + '.';
            }
            open += name;
            open += '\n' + spaces(depth) + '[';
            open += attributesToString(attribs, attributeAlias);
            open += ']\n' + spaces(depth) + '[';
            fragments.push(open);
        },
        ontext: text => {
            // TODO: Add support for html tags inside text
            if (text.trim().length) {
                const tag = htmlAlias.length ? `${htmlAlias}.text` : 'text';
                fragments.push(` ${tag} "${text.trim()}"`);
            }
        },
        onclosetag: _ => {
            let close = '';
            switch (previousFragment(fragments)) {
                case Fragment.Close:
                    close += '\n' + spaces(depth);
                    break;
                case Fragment.Text:
                    close += ' ';
                    break;
            }
            close += ']';
            depth--;
            fragments.push(close);
        },
    }, { decodeEntities: true, lowerCaseAttributeNames: true });
    parser.write(html);
    parser.end();
    return fragments.join('');
};
exports.default = convert;
